var React = require('react');
var Router = require('react-router');
var BS = require('react-bootstrap');
var Link = Router.Link;
var _ = require('underscore');

var BenchmarkStore = require('../stores/BenchmarkStore');
var BenchmarkActions = require('../actions/BenchmarkActions');
var ComparisonStore = require('../stores/ComparisonStore');
var ComparisonActions = require('../actions/ComparisonActions');
var Header = require('./Header.react');
var Graph = require('./Graph.react');
var ActionButton = require('./ActionButton.react');

var Utils = require('../utils/Utils');
var moment = require('moment');


var BenchmarkCompare = React.createClass({
  contextTypes: {
    router: React.PropTypes.func
  },

  getInitialState: function() {
    var from = this.context.router.getCurrentQuery().from;
    var to = this.context.router.getCurrentQuery().to;
    var id = [from, to].sort().join('-');
    return {
      from: from,
      to: to,
      id: id,
      comparison: null,
      graphEditorVisible: false
    }
  },

  componentWillMount: function() {
    ComparisonActions.refresh(this.state.id);
    for (var uuid of [this.state.from, this.state.to]) {
      BenchmarkActions.refreshMetrics(uuid);
    }
  },

  componentDidMount: function() {
    ComparisonStore.addChangeListener(this._onChange);
  },

  componentWillUnmount: function() {
    ComparisonStore.removeChangeListener(this._onChange);
  },

  _onChange: function() {
    this.setState({
      comparison: ComparisonStore.get(this.state.id)
    });
  },

  _onGraphSave: function() {
    ComparisonActions.refresh(this.state.id);
  },

  _toggleGraphEditor: function(event, show) {
    this.setState({
      graphEditorVisible: show
    });
    if (event) {
      event.preventDefault();
    }
  },

  _getHeadings: function(from, to) {
    if (from.environment.uuid !== to.environment.uuid) {
      return [from.environment.name, to.environment.name];
    }

    if (from.service !== to.service || from.name !== to.name) {
      return [`${from.service}:${from.name}`, `${to.service}:${to.name}`];
    }

    var fromStart = moment(from.started),
      toStart = moment(to.started),
      day = 'YYYY-MM-DD',
      time = 'HH:MM:SS',
      fromStartDay = fromStart.format(day),
      toStartDay = toStart.format(day);

    if (fromStartDay === toStartDay) {
      return [fromStart.format(time), toStart.format(time)];
    }

    return [fromStartDay, toStartDay];
  },

  render: function() {
    var from = BenchmarkStore.get(this.state.from),
      to = BenchmarkStore.get(this.state.to),
      from_results = this.ensureResults(from),
      to_results = this.ensureResults(to);

    var [fromHeader, toHeader] = this._getHeadings(from, to);

    return (
      <div>
        <Header>
          <ActionButton>
            <BS.MenuItem
              onClick={(e) => this._toggleGraphEditor(e, true)}
            >Add Graph</BS.MenuItem>
            <BS.MenuItem>
              <a href={BenchmarkStore.getSettings().graphite_url}
                  target="_blank"
              >Graphite Browser</a>
            </BS.MenuItem>
          </ActionButton>
          <h1>Benchmark Comparison</h1>
          {this.state.graphEditorVisible &&
            <Graph.GraphEditor
              actions={[from, to]}
              saveUrl={`/api/comparisons/${this.state.id}/graphs`}
              onSave={this._onGraphSave}
              onRequestHide={() => this._toggleGraphEditor(null, false)}
            />
          }
        </Header>
        <BS.Grid>
          <table className="table compare-table">
            <thead>
              <tr>
                <th></th>
                <th>
                  <h3>
                    <Link to="action" params={from}>
                      {fromHeader}
                    </Link>
                  </h3>
                </th>
                <th>
                  <h3>
                    <Link to="action" params={to}>
                      {toHeader}
                    </Link>
                  </h3>
                </th>
              </tr>
            </thead>
            <tbody>
              <tr>
                <td>UUID</td>
                <td>{from.uuid}</td>
                <td>{to.uuid}</td>
              </tr>
              <tr>
                <td>Status</td>
                <td>{from.status}</td>
                <td>{to.status}</td>
              </tr>
              <tr>
                <td>Unit</td>
                <td>{from.unit}</td>
                <td>{to.unit}</td>
              </tr>
              <tr>
                <td>Started</td>
                <td>{from.started}</td>
                <td>{to.started}</td>
              </tr>
              <tr>
                <td>Duration</td>
                <td>{from.duration}</td>
                <td>{to.duration}</td>
              </tr>
              {_.union(_.keys(from_results),
                       _.keys(to_results)).map(function(k) {
                return k == 'raw' ? null : (
                  <tr key={k}>
                    <td>{k}</td>
                    <td>{Utils.formatResult(from_results[k])}</td>
                    <td>{Utils.formatResult(to_results[k])}</td>
                  </tr>
                );
              })}
            </tbody>
          </table>

          {this.state.comparison &&
          <Graph.GraphList
            graphs={this.state.comparison.graphs}
            onDelete={(graph_id) => {
              ComparisonActions.deleteGraph(this.state.id, graph_id);
            }}
          />
          }

          <UnitCompare fromAction={from} toAction={to} />
        </BS.Grid>
      </div>
    );
  },

  ensureResults: function(benchmark) {
    return (
      benchmark.output && benchmark.output.results ?
        benchmark.output.results :
        {}
    );
  }
});

var UnitCompare = React.createClass({
  propTypes: {
    fromAction: React.PropTypes.object.isRequired,
    toAction: React.PropTypes.object.isRequired
  },

  render: function() {
    return (
      <div>
        <ParametersCompare from={this.props.fromAction} to={this.props.toAction} />
        <ProfileCompare from={this.props.fromAction} to={this.props.toAction} />
      </div>
    );
  }
});

var ActionLink = React.createClass({
  propTypes: {
    action: React.PropTypes.object.isRequired
  },

  render: function() {
    return (
      <Link to="action" params={this.props.action}>
        {this.props.action.uuid}
      </Link>
    );
  }
});

var ProfileCompare = React.createClass({
  propTypes: {
    from: React.PropTypes.object.isRequired,
    to: React.PropTypes.object.isRequired
  },

  getInitialState: function() {
    return {
      fromProfile: null,
      toProfile: null
    };
  },

  handleFromProfileChange: function(profile) {
    this.setState({
      fromProfile: profile
    });
  },

  handleToProfileChange: function(profile) {
    this.setState({
      toProfile: profile
    });
  },

  render: function() {
    var from = this.state.fromProfile || {};
    var to = this.state.toProfile || {};
    var from_hardware = (from.hardware || {})['lshw'] || {};
    var to_hardware = (to.hardware || {})['lshw'] || {};
    from.packages = from.packages || {};
    to.packages = to.packages || {};
    var hwKeys = _.union(_.keys(from_hardware), _.keys(to_hardware));
    var pkgKeys = _.union(_.keys(from.packages), _.keys(to.packages));
    var hwDiffs = _.filter(hwKeys, function(k) {
      return from_hardware[k] !== to_hardware[k];
    });
    var hwDiffHtml = hwDiffs.length ? (
      <div key="hwDiffs">
        <h2>Hardware Differences</h2>
        <table className='table compare-table'>
          <tbody>
          {hwDiffs.sort().map(function(k) {
            return (
              <tr key={k}>
                <td>{k}</td>
                <td>{String(typeof from_hardware[k] != 'undefined' ? from_hardware[k] : '')}</td>
                <td>{String(typeof to_hardware[k] != 'undefined' ? to_hardware[k] : '')}</td>
              </tr>
            );
          })}
          </tbody>
        </table>
      </div>
    ) : null;

    var pkgDiffHtml = pkgKeys.sort().map(function(k) {
      var fromPkgs = _.indexBy(from.packages[k], 'name');
      var toPkgs = _.indexBy(to.packages[k], 'name');
      var pkgNames = _.union(_.keys(fromPkgs), _.keys(toPkgs));
      var pkgDiffs = _.filter(pkgNames, function(name) {
        return (fromPkgs[name] || {}).version !== (toPkgs[name] || {}).version;
      });
      return pkgDiffs.length ? (
        <div key={k}>
          <h2>{k}</h2>
          <table className='table compare-table'>
            <tbody>
            {pkgDiffs.sort().map(function(pkgName) {
              return (
                <tr key={pkgName}>
                  <td>{pkgName}</td>
                  <td>{(fromPkgs[pkgName] || {}).version}</td>
                  <td>{(toPkgs[pkgName] || {}).version}</td>
                </tr>
              );
            })}
            </tbody>
          </table>
        </div>
      ) : null;
    });

    return (
      <div>
        <table className='table compare-table'>
          <tbody>
            <tr>
              <td></td>
              <td>
                <Utils.UnitProfileChooser
                  action={this.props.from}
                  onChange={this.handleFromProfileChange}
                />
              </td>
              <td>
                <Utils.UnitProfileChooser
                  action={this.props.to}
                  onChange={this.handleToProfileChange}
                />
              </td>
            </tr>
          </tbody>
        </table>
        {hwDiffHtml}
        {pkgDiffHtml}
      </div>
    );

  }
});

var ParametersCompare = React.createClass({
  propTypes: {
    from: React.PropTypes.object.isRequired,
    to: React.PropTypes.object.isRequired
  },

  render: function() {
    var from = this.props.from.action.parameters || {};
    var to = this.props.to.action.parameters || {};
    var paramKeys = _.union(_.keys(from), _.keys(to));
    var paramDiffs = _.filter(paramKeys, function(k) {
      return from[k] !== to[k];
    });

    if (paramDiffs.length === 0) {
      return null;
    }

    return (
      <div>
        <h2>Parameter Differences</h2>
        <table className='table compare-table'>
          <tbody>
          {paramDiffs.sort().map(function(k) {
            return (
              <tr key={k}>
                <td>{k}</td>
                <td>{String(typeof from[k] != 'undefined' ? from[k] : '')}</td>
                <td>{String(typeof to[k] != 'undefined' ? to[k] : '')}</td>
              </tr>
            );
          })}
          </tbody>
        </table>
      </div>
    );

  }
});

module.exports = BenchmarkCompare;
